# -*- coding: utf-8 -*-

# pylint: disable=too-few-public-methods

"""Common widgets and their bases."""

__all__ = ['IconButton']

import typing
from qtpy import QtWidgets, QtCore
from anaconda_navigator.static import images
from anaconda_navigator import widgets

if typing.TYPE_CHECKING:
    import typing_extensions


class IconButton(QtWidgets.QPushButton):
    """Common control for icon-only buttons."""


class WarningIcon(QtWidgets.QLabel):
    """
    Icon with exclamation sign.

    Used by :class:`~WarningLabel` and its derivatives.
    """


class ErrorIcon(QtWidgets.QLabel):
    """
    Icon with exclamation sign in circle.
    """


class InfoIcon(QtWidgets.QLabel):
    """
    Icon with exclamation sign in circle.
    """


class LabelWithIcon(QtWidgets.QFrame):
    """
    Combined label with icon at the beginning.

    Icon is shown only when the main text is not empty.
    """

    sig_button_clicked = QtCore.Signal()

    def __init__(  # pylint: disable=too-many-arguments
            self,
            icon: typing.Optional[widgets.QLabel] = None,
            text: str = '',
            tooltip: str = '',
            button: str = '',
            alignment: typing.Any = QtCore.Qt.AlignTop,
    ) -> None:
        """Initialize new :class:`~WarningLabel` instance."""
        super().__init__()

        self.__icon: 'typing_extensions.Final[widgets.QLabel]' = icon
        self.__icon.setVisible(bool(text))

        self.__content: 'typing_extensions.Final[widgets.LabelBase]' = widgets.LabelBase()
        self.__content.setText(text)
        self.__content.setToolTip(tooltip)
        self.__content.setWordWrap(True)

        self.__button: 'typing_extensions.Final[widgets.LabelBase]' = widgets.LabelBase()
        self.__button.setText(button)
        self.__button.sig_clicked.connect(self.sig_button_clicked)

        self.__layout: 'typing_extensions.Final[QtWidgets.QHBoxLayout]' = QtWidgets.QHBoxLayout()
        self.__layout.addWidget(self.__icon, 0, alignment)
        self.__layout.addWidget(self.__content, 1, alignment)
        if button:
            self.__layout.addWidget(self.__button, 0, alignment)
        self.__layout.setContentsMargins(0, 0, 0, 0)
        self.__layout.setSpacing(4)

        self.setLayout(self.__layout)

    @property
    def text(self) -> str:  # noqa: D401
        """Content of the main label."""
        return self.__content.text()

    @text.setter
    def text(self, value: str) -> None:
        """Update `text` value."""
        self.__content.setText(value)
        self.__icon.setVisible(bool(value))


class WarningLabel(LabelWithIcon):
    """
    Combined label with warning icon at the beginning.

    Icon is shown only when the main text is not empty.
    """

    def __init__(
            self,
            text: str = '',
            tooltip: str = '',
            button: str = '',
            alignment: typing.Any = QtCore.Qt.AlignTop,
    ) -> None:
        """Initialize new :class:`~WarningLabel` instance."""
        super().__init__(icon=WarningIcon(), text=text, tooltip=tooltip, button=button, alignment=alignment)


class WarningBlock(WarningLabel):
    """Same as :class:`~WarningLabel`, but with colored background."""

    def __init__(
            self,
            text: str = '',
            tooltip: str = '',
            button: str = '',
            alignment: typing.Any = QtCore.Qt.AlignTop,
    ) -> None:
        """Initialize new :class:`~WarningBlock` instance."""
        super().__init__(text=text, tooltip=tooltip, button=button, alignment=alignment)


class ErrorLabel(LabelWithIcon):
    """
    Combined label with error icon at the beginning.

    Icon is shown only when the main text is not empty.
    """

    def __init__(
            self,
            text: str = '',
            tooltip: str = '',
            button: str = '',
            alignment: typing.Any = QtCore.Qt.AlignTop,
    ) -> None:
        """Initialize new :class:`~ErrorLabel` instance."""
        super().__init__(icon=ErrorIcon(), text=text, tooltip=tooltip, button=button, alignment=alignment)


class ErrorBlock(ErrorLabel):
    """Same as :class:`~ErrorLabel`, but with colored background."""

    def __init__(
            self,
            text: str = '',
            tooltip: str = '',
            button: str = '',
            alignment: typing.Any = QtCore.Qt.AlignTop,
    ) -> None:
        """Initialize new :class:`~ErrorBlock` instance."""
        super().__init__(text=text, tooltip=tooltip, button=button, alignment=alignment)


class InfoLabel(LabelWithIcon):
    """
    Combined label with info icon at the beginning.

    Icon is shown only when the main text is not empty.
    """

    def __init__(
            self,
            text: str = '',
            tooltip: str = '',
            button: str = '',
            alignment: typing.Any = QtCore.Qt.AlignTop,
    ) -> None:
        """Initialize new :class:`~InfoLabel` instance."""
        super().__init__(icon=InfoIcon(), text=text, tooltip=tooltip, button=button, alignment=alignment)


class InfoBlock(InfoLabel):
    """Same as :class:`~InfoLabel`, but with colored background."""

    def __init__(
            self,
            text: str = '',
            tooltip: str = '',
            button: str = '',
            alignment: typing.Any = QtCore.Qt.AlignTop,
    ) -> None:
        """Initialize new :class:`~InfoBlock` instance."""
        super().__init__(text=text, tooltip=tooltip, button=button, alignment=alignment)


class TeamEditionServerAlert(QtWidgets.QWidget):
    """Show combined labels with icon about connectivity problems to TE Server."""

    def __init__(self) -> None:
        super().__init__()
        self.error_label: 'typing_extensions.Final[ErrorBlock]' = ErrorBlock(
            text=(
                '<b>Anaconda Anaconda Server server cannot be reached.</b> '
                'Some actions may not work as expected until connection is restored.'
            ),
            alignment=QtCore.Qt.AlignVCenter,
        )
        self.info_label: 'typing_extensions.Final[InfoBlock]' = InfoBlock(
            text='<b>Connection to Anaconda Anaconda Server server restored.</b>',
            button=f'<img src="{images.BLOCK_CLOSE_ICON_PATH}" height="16" width="16">',
            alignment=QtCore.Qt.AlignVCenter,
        )
        self.__timer: 'typing_extensions.Final[QtCore.QTimer]' = QtCore.QTimer()

        layout: 'typing_extensions.Final[QtWidgets.QVBoxLayout]' = QtWidgets.QVBoxLayout()
        layout.addWidget(self.error_label)
        layout.addWidget(self.info_label)

        self.info_label.sig_button_clicked.connect(self.hide_all)

        self.setLayout(layout)
        self.hide_all()

    def show_info(self, limit: typing.Optional[int] = 5000) -> None:
        """Show info label for `limit` seconds"""
        if not self.error_label.isVisible():
            self.error_label.hide()
            return

        self.show()
        self.error_label.hide()
        self.info_label.show()
        if limit:
            self.__timer.singleShot(limit, self.hide_all)

    def show_error(self) -> None:
        """Show error label"""
        self.show()
        self.error_label.show()
        self.info_label.hide()

    def hide_all(self) -> None:
        """Hide all labels."""
        self.hide()
        self.error_label.hide()
        self.info_label.hide()



# -*- coding: utf-8 -*-

"""Collection of utility components to use by APIs."""

from __future__ import annotations

__all__ = ['normalize_certificate']

import os
import typing


def normalize_certificate(value: typing.Union[None, bool, str]) -> typing.Union[None, bool, str]:
    """Check if certificate value is valid and fix it if required."""
    if isinstance(value, str) and (not os.path.exists(value)):
        return True
    return value



# -*- coding: utf-8 -*-

"""Core interfaces for Conda error solvers."""

from __future__ import annotations

__all__ = ['Solver', 'SimpleSolver', 'SolvedError', 'SolverCollection', 'POOL']

import abc
import typing
from anaconda_navigator.utils import solvers as common_solvers

if typing.TYPE_CHECKING:
    import typing_extensions
    from .. import types as conda_types

    TConError = typing.TypeVar('TConError', bound='conda_types.CondaErrorOutput', contravariant=True)

    class SolverFunction(typing_extensions.Protocol[TConError]):  # pylint: disable=too-few-public-methods
        """General form of a function, that could be used as a solver."""

        def __call__(self, error: TConError) -> str:
            """
            Solve Conda error.

            :param error: Description of the error to solve.
            :return: Message about what was fixed.
            """


TInvError = typing.TypeVar('TInvError', bound='conda_types.CondaErrorOutput')


class Solver(typing.Generic[TInvError], metaclass=abc.ABCMeta):  # pylint: disable=too-few-public-methods
    """
    Abstract base for all Conda error solvers.

    :param kwargs: Exact values to search for in the error body.
    """

    __slots__ = ('__arguments',)

    def __init__(self, **kwargs: typing.Any) -> None:
        """Initialize new :class:`~ErrorSolver` instance."""
        self.__arguments: 'typing_extensions.Final[typing.Mapping[str, typing.Any]]' = dict(kwargs)

    def solve(self, error: 'conda_types.CondaErrorOutput') -> typing.Optional[str]:
        """
        Attempt solution of the `error`.

        :param error: Description of the error to solve.
        :return: Message about what was fixed. Might be :code:`None` if the solver is not applicable.
        """
        if self._applicable(error=error):
            return self._solve(error=typing.cast(TInvError, error))
        return None

    def _applicable(self, error: 'conda_types.CondaErrorOutput') -> bool:
        """
        Check if this solver could be used to solve the `error`.

        This method might be overwritten to add custom checks.

        :param error: Description of the error to solve.
        :return: Check result.
        """
        return all(
            error.get(key, None) == value
            for key, value in self.__arguments.items()
        )

    @abc.abstractmethod
    def _solve(self, error: TInvError) -> str:
        """
        Solve Conda error.

        :param error: Description of the error to solve.
        :return: Message about what was fixed.
        """


class SimpleSolver(typing.Generic[TInvError], Solver[TInvError]):  # pylint: disable=too-few-public-methods
    """
    Custom :class:`~ErrorSolver`, which uses external function to solve an error.

    :param __function__: Function to use for error solving.
    :param kwargs: Exact values to search for in the error body.
    """

    __slots__ = ('__function',)

    def __init__(self, __function__: 'SolverFunction[TInvError]', **kwargs: typing.Any) -> None:
        """Initialize new :class:`~SimpleErrorSolver` instance."""
        super().__init__(**kwargs)
        self.__function: 'typing_extensions.Final[SolverFunction[TInvError]]' = __function__

    def _solve(self, error: TInvError) -> str:
        """
        Solve Conda error.

        :param error: Description of the error to solve.
        :return: Message about what was fixed.
        """
        return self.__function(error=error)


class SolvedError(typing.NamedTuple):
    """
    Record for the error, that was solved.

    :param error: Description of the error that was solved.
    :param message: Message about what was fixed.
    """

    error: 'conda_types.CondaErrorOutput'
    message: str


class SolverCollection(common_solvers.SolverCollection[Solver[typing.Any]]):
    """Collection of solvers."""

    __slots__ = ('__errors',)

    def __init__(self) -> None:
        """Initialize new :class:`~SolverCollection` instance."""
        super().__init__()
        self.__errors: 'typing_extensions.Final[typing.List[SolvedError]]' = []

    def errors(self) -> typing.Iterator[SolvedError]:
        """
        Iterate through errors that were solved by solvers from this pool.

        Each error description is removed as soon as it is retrieved with this method.
        """
        while self.__errors:
            yield self.__errors.pop(0)

    @typing.overload
    def register_function(
            self,
            function: 'SolverFunction[TInvError]',
            *,
            filters: typing.Optional[typing.Mapping[str, typing.Any]] = None,
            tags: typing.Union[str, typing.Iterable[str]] = (),
            unique_tags: typing.Union[str, typing.Iterable[str]] = (),
    ) -> 'SolverFunction[TInvError]':
        """Register function as a solver."""

    @typing.overload
    def register_function(
            self,
            function: None = None,
            *,
            filters: typing.Optional[typing.Mapping[str, typing.Any]] = None,
            tags: typing.Union[str, typing.Iterable[str]] = (),
            unique_tags: typing.Union[str, typing.Iterable[str]] = (),
    ) -> typing.Callable[['SolverFunction[TInvError]'], 'SolverFunction[TInvError]']:
        """Register function as a solver."""

    def register_function(self, function=None, filters=None, tags=(), unique_tags=()):
        """
        Register new solver.

        This method can be used as a decorator, as well as direct function. See :meth:`~SolverCollection.register` for
        more details.

        :param function: Function that should be registered.
        :param filters: Values that should be in the error body in order to apply current fix.
        :param tags: Optional collection of common tags.
        :param unique_tags: Optional collection of tags, that should be unique only for this particular solver.
        """
        if filters is None:
            filters = {}

        def wrapper(item: 'SolverFunction[TInvError]') -> 'SolverFunction[TInvError]':
            self.register(
                solver=SimpleSolver[TInvError](item, **filters),
                tags=tags,
                unique_tags=unique_tags,
            )
            return item

        if function is None:
            return wrapper
        return wrapper(function)

    def solve(
            self,
            error: 'conda_types.CondaErrorOutput',
            *,
            tags: typing.Union[None, str, typing.Iterable[str]] = None,
    ) -> typing.Optional[SolvedError]:
        """
        Detect and solve issues for the `context`.

        :param error: Conda error to solve.
        :param tags: Limit to issue solvers with specific tags.
        :return: Iterator of details about solved issues.
        """
        solver_record: common_solvers.SolverRecord[Solver[typing.Any]]
        for solver_record in self.only(tags=tags):
            message: typing.Optional[str] = solver_record.solver.solve(error)
            if message is not None:
                solved_error: SolvedError = SolvedError(error=error, message=message)
                self.__errors.append(solved_error)
                return solved_error
        return None


POOL: 'typing_extensions.Final[SolverCollection]' = SolverCollection()



# -*- coding: utf-8 -*-

"""External Datalore web application."""

__all__ = ['DataloreApp']

import typing
from anaconda_navigator.static import images
from . import base

if typing.TYPE_CHECKING:
    from anaconda_navigator.config import user as user_config


class DataloreApp(base.BaseWebApp):  # pylint: disable=too-few-public-methods
    """Shortcut for JetBrains Datalore web application."""

    def __init__(self, config: 'user_config.UserConfig') -> None:
        """Initialize new :class:`DataloreApp` instance."""
        super().__init__(
            app_name='datalore',
            display_name='Datalore',
            description=(
                'Online Data Analysis Tool with smart coding assistance by JetBrains. Edit and run your Python '
                'notebooks in the cloud and share them with your team.'
            ),
            image_path=images.DATALORE_ICON_1024_PATH,
            url='http://www.anaconda.com/datalore_navigator',
            is_available=True,
            config=config,
        )



# -*- coding: utf-8 -*-

"""Navigator Exceptions and Exception handling module."""

import os
import tempfile
from traceback import format_exc
import typing
from urllib.request import pathname2url
import webbrowser
from anaconda_navigator.widgets.dialogs import MessageBoxError
from anaconda_navigator.utils.logs import logger


def display_qt_error_box(error, traceback):
    """Display a Qt styled error message box."""
    text = f'An unexpected error occurred on Navigator start-up<br>{error}'
    trace = f'{traceback}'
    msg_box = MessageBoxError(
        title='Navigator Start Error',
        text=text,
        error=trace,
        report=False,  # Disable reporting on github
        learn_more=None,
    )
    msg_box.setFixedWidth(600)
    return msg_box.exec_()


def display_browser_error_box(error, traceback):
    """Display a new browser window with an error description."""
    template = '''
    <html>
    <head>
      <title>Navigator Error</title>
    </head>
    <body>
      <div>
        <h1>Navigator Error</h1>
        <p>An unexpected error occurred on Navigator start-up</p>
        <h2>Report</h2>
        <p>Please report this issue in the anaconda
          <a href="https://github.com/continuumio/anaconda-issues">
            issue tracker
          </a>
        </p>
      </div>
      <div>
        <h2>Main Error</h2>
        <p><pre>{error}</pre></p>
        <h2>Traceback</h2>
        <p><pre>{trace}</pre></p>
      </div>
    </body>
    </html>
    '''

    file_descriptor: int
    file_path: str
    file_descriptor, file_path = tempfile.mkstemp(suffix='.html')

    file_stream: typing.TextIO
    with os.fdopen(file_descriptor, 'wt', encoding='utf-8') as file_stream:
        file_stream.write(template.format(error=error, trace=traceback))

    url = f'file:{pathname2url(file_path)}'

    webbrowser.open_new_tab(url)


def exception_handler(func, *args, **kwargs):
    """Handle global application exceptions and display information."""
    try:
        return_value = func(*args, **kwargs)
        if isinstance(return_value, int):
            return return_value
    except Exception as e:  # pylint: disable=broad-except,invalid-name
        return handle_exception(e)
    return None


def handle_exception(error):
    """This will provide a dialog for the user with the error found."""
    traceback = format_exc()
    logger.critical(error, exc_info=True)
    try:
        display_qt_error_box(error, traceback)
    except Exception as exc:  # pylint: disable=broad-except
        logger.exception(exc)
        display_browser_error_box(error, traceback)



#!/opt/concourse/worker/volumes/live/a76d46df-8008-46e2-763c-d3f400318082/volume/anaconda-navigator_1648477386214/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/a76d46df-8008-46e2-763c-d3f400318082/volume/anaconda-navigator_1648477386214/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])



# -*- coding: utf-8 -*-

"""Exceptions that could be raised by :mod:`~anaconda-navigator.utils.conda` utilities."""

from __future__ import annotations

__all__ = ['CondaError']

import typing

if typing.TYPE_CHECKING:
    import typing_extensions
    from . import types as conda_types


class CondaError(BaseException):
    """
    General error raised by the Conda process.

    :param error: Content of the Conda response.
    """

    def __init__(self, error: 'conda_types.CondaErrorOutput') -> None:
        """Initialize new :class:`~CondaError` instance."""
        super().__init__()
        self.__error: 'typing_extensions.Final[conda_types.CondaErrorOutput]' = error

    @property
    def error(self) -> 'conda_types.CondaErrorOutput':
        """Content of the Conda error output."""
        return self.__error
